// kllinear.cc
//
// Author: allauzen@google.com (Cyril Allauzen)
//
// \file
// Generate a linear kernel from a feature map.

#include "kernel/lib/kernel.h"
#include "kernel/lib/feature-weighted-kernel.h"
#include "kernel/lib/sparse-vector.h"
#include "kernel/lib/dense-vector.h"

using namespace kernel;
using namespace fst;

DEFINE_bool(normalize, false, "normalize kernel: k[x,y] / sqrt(k[x,x] * k[y,y])");
DEFINE_bool(cache_feature, false, "cache RKHS feature computation");
DEFINE_bool(cache_norm, false, "cache RKHS norm computation");
DEFINE_bool(cache_value, false, "cache kernel value computation");


int main(int argc, char** argv) {
  string usage = "Create an linear kernel.\n\n  Usage: ";
  usage += argv[0];
  usage += " dataset.fstlist > rational.kar \n";
  if (argc == 1) {
    cerr << argv[0] << ": use --help for usage" << endl;
    return 1;
  }

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc < 2 || argc > 3) {
    ShowUsage();
    return 1;
  }


  DataSet<SparseVector<double> > *data =
      new DataSet<SparseVector<double> >(string(argv[1]));

  size_t max_index = 0;
  for (size_t i = 0; i < data->Size(); ++i)
    if ((*data)[i].MaxIndex() > max_index)
      max_index = (*data)[i].MaxIndex();

  DenseVector<double> weights;
  for (size_t i = 0; i <= max_index; ++i)
    weights.AddElement(i, 1.0);

  FeatureWeightedKernel<SparseVector<double> > kernel(
      weights,
      FLAGS_normalize,
      (FLAGS_cache_feature ? kCacheFeature : 0) |
      (FLAGS_cache_norm ? kCacheNorm : 0) |
      (FLAGS_cache_value ? kCacheValue : 0));

  kernel.SetData(data);
  kernel.Write(argc > 2 ? string(argv[2]) : string());
}
