// klmismatchfst.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Generate a mismatch counting fst

#include <fst/fstlib.h>
#include "kernel/lib/mismatch-fst.h"
#include "kernel/lib/rational-kernel.h"

DEFINE_int32(opt, -1, "rational function optimization level 0 to 5, -1 for default");
DEFINE_bool(normalize, true, "normalize rational kernel: k[x,y] / sqrt(k[x,x] * k[y,y])");
DEFINE_int32(k, -1, "k, subsequence of lenght k");
DEFINE_int32(m, -1, "m, at most m mismatches");
DEFINE_int32(sigma, -1, "alphabet size (i.e. max label)");

using namespace kernel;
using namespace fst;

typedef VectorFst<LogArc> LogVectorFst;


int main(int argc, char** argv) {
  string usage = "Generate a (k,m)-mismatch kernel (k > m).\n\n  Usage: ";
  usage += argv[0];
  usage += " dataset.fstlist > out.fst \n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc < 2 || argc > 3) {
    ShowUsage();
    return 1;
  }

  if ((FLAGS_k == -1) || (FLAGS_m == -1)) {
    LOG(FATAL) << "klmismatch: k and m values need to be specified \n";
  }
  if (FLAGS_sigma == -1) {
    LOG(FATAL) << "klmismatch: value of larger symbol need to be specified "
               << "with --sigma=n\n";
  }

  LogVectorFst fst;
  Mismatch(&fst, FLAGS_k, FLAGS_m, FLAGS_sigma);
  ArcSort(&fst, ILabelCompare<LogArc>());

  RationalKernel<LogArc> kernel(fst, FLAGS_opt, FLAGS_normalize);
  DataSet<LogFst> *data = new DataSet<LogFst>(string(argv[1]));
  kernel.SetData(data);
  kernel.Write(argc > 2 ? string(argv[2]) : string());
}
